searchData={"items":[{"type":"behaviour","title":"Kungfuig","doc":"`Kungfuig` provides a pluggable drop-in support for live configurations with change notification capabilities.","ref":"Kungfuig.html"},{"type":"behaviour","title":"Overview - Kungfuig","doc":"Kungfuig (pronounced: [ˌkʌŋˈfig]) is a comprehensive configuration management system for Elixir applications\nthat provides:\n\n* Live configuration updates from multiple sources\n* Callback notifications when configurations change\n* Configuration validation\n* Extensible backend system for custom configuration sources\n* Supervisor-managed configuration processes\n\nThis behaviour defines the dynamic config provider. In most cases, you don't need to implement\nthis behaviour directly. Instead, implement one or more backends (see `Kungfuig.Backend`)\nand pass them, optionally parametrized, as `workers:` to `start_link/1` as shown below:\n\n```elixir\n{:ok, pid} =\n  Kungfuig.start_link(\n    workers: [\n      {Kungfuig.Backends.EnvTransform,\n       interval: 100,\n       validator: Kungfuig.Validators.EnvTransform,\n       callback: {self(), {:info, :updated}},\n       callback: Kungfuig.Test.CallbackAlert}\n    ]\n  )\n```\n\nBy default, `Kungfuig` starts two backends:\n* Application environment backend (`Application.get_all_env/1`) \n* System environment backend (for environment variables)","ref":"Kungfuig.html#module-overview"},{"type":"behaviour","title":"Architecture - Kungfuig","doc":"Kungfuig operates using a supervisor tree pattern:\n\n1. A supervisor (`Kungfuig.Supervisor`) manages the configuration process\n2. A manager process (`Kungfuig.Manager`) supervises the backend workers\n3. Backend workers (`Kungfuig.Backend` implementations) poll their respective configuration sources\n4. A blender process (`Kungfuig.Blender`) combines configurations from all backends\n5. Callbacks are triggered when configurations change","ref":"Kungfuig.html#module-architecture"},{"type":"behaviour","title":"Custom Configuration Sources - Kungfuig","doc":"If you need to get configuration updates from sources other than the default ones,\nsuch as JSON/YAML files, Redis, databases, or any external service, you can create\nyour own backends by implementing the `Kungfuig.Backend` behaviour.\n\nSee `Kungfuig.Backend` for reference or check the \n[Usage](https://hexdocs.pm/kungfuig/readme.html#using) section in the README for an example\nusing MySQL.","ref":"Kungfuig.html#module-custom-configuration-sources"},{"type":"behaviour","title":"Validation - Kungfuig","doc":"Since v0.3.0, Kungfuig supports validation through the `Kungfuig.Validator` behaviour,\nwhich by default uses `NimbleOptions` for schema validation. Validators can be specified:\n\n* Per backend (applying only to that backend's configuration)\n* Globally (applying to the entire configuration)","ref":"Kungfuig.html#module-validation"},{"type":"behaviour","title":"Callbacks - Kungfuig","doc":"When a configuration changes, Kungfuig notifies interested parties through callbacks.\nMultiple callback mechanisms are supported, as detailed in the `t:callback/0` type.","ref":"Kungfuig.html#module-callbacks"},{"type":"behaviour","title":"Named Instances - Kungfuig","doc":"Since v0.4.0, Kungfuig supports multiple named instances, allowing different parts of your\napplication to use separate configuration managers.","ref":"Kungfuig.html#module-named-instances"},{"type":"behaviour","title":"Examples - Kungfuig","doc":"Basic usage:\n\n```elixir\n# Start Kungfuig with default options\nKungfuig.start_link()\n\n# Get all configuration\nKungfuig.config()\n#⇒ %{env: %{kungfuig: []}, system: %{}}\n\n# Get configuration from a specific backend\nKungfuig.config(:env)\n#⇒ %{kungfuig: []}\n\n# Update application environment and see the change\nApplication.put_env(:kungfuig, :foo, 42)\nKungfuig.config(:env)\n#⇒ %{kungfuig: [foo: 42]}\n```\n\nUsing callbacks:\n\n```elixir\n# Using a process message as callback\nKungfuig.start_link(\n  workers: [\n    {Kungfuig.Backends.Env, callback: {self(), {:info, :config_updated}}}\n  ]\n)\n\n# Using a module that implements Kungfuig.Callback\ndefmodule MyApp.ConfigHandler do\n  @behaviour Kungfuig.Callback\n  \n  @impl true\n  def handle_config_update(config) do\n    IO.puts(\"Configuration updated: \" <> inspect(config))\n  end\nend\n\nKungfuig.start_link(\n  workers: [\n    {Kungfuig.Backends.Env, callback: MyApp.ConfigHandler}\n  ]\n)\n```\n\nUsing named instances (since v0.4.0):\n\n```elixir\n# Start a named Kungfuig instance\nKungfuig.start_link(name: MyApp.Config)\n\n# Query the named instance\nKungfuig.config(:env, MyApp.Config)\n```\n\nUsing immediate validation (since v0.4.2):\n\n```elixir\n# Start with imminent validation for immediate config processing\nKungfuig.start_link(\n  workers: [\n    {Kungfuig.Backends.Env, imminent: true, validator: MyApp.ConfigValidator}\n  ]\n)\n```","ref":"Kungfuig.html#module-examples"},{"type":"type","title":"Kungfuig.callback/0","doc":"The callback to be used for subscribing to configuration updates.\n\nKungfuig supports several callback mechanisms:\n\n* **Module implementing `Kungfuig.Callback` behaviour**:\n  ```elixir\n  defmodule MyApp.ConfigHandler do\n    @behaviour Kungfuig.Callback\n    \n    @impl true\n    def handle_config_update(config) do\n      # Handle configuration update\n      :ok\n    end\n  end\n  ```\n\n* **Process identifier (PID)**: The process will receive a `{:kungfuig_update, config}` message\n  ```elixir\n  # In a GenServer\n  def handle_info({:kungfuig_update, config}, state) do\n    # Handle configuration update\n    {:noreply, state}\n  end\n  ```\n\n* **Module-function tuple**: A tuple `{module, function}` where the function accepts a single \n  argument (the configuration)\n  ```elixir\n  defmodule MyApp.ConfigHandler do\n    def on_config_update(config) do\n      # Handle configuration update\n    end\n  end\n  \n  # In your Kungfuig setup\n  callback: {MyApp.ConfigHandler, :on_config_update}\n  ```\n\n* **Anonymous function**: A function that takes one argument (the configuration)\n  ```elixir\n  callback: fn config -> \n    # Handle configuration update\n    :ok \n  end\n  ```\n\n* **GenServer interaction tuple**: A tuple specifying a GenServer and how to interact with it\n  ```elixir\n  # Send a cast to a GenServer\n  callback: {MyGenServer, {:cast, :config_updated}}\n  \n  # Inside the GenServer\n  def handle_cast({:config_updated, config}, state) do\n    # Handle configuration update\n    {:noreply, state}\n  end\n  ```","ref":"Kungfuig.html#t:callback/0"},{"type":"type","title":"Kungfuig.config/0","doc":"The config map to be updated and exposed through callbacks.\n\nThis is a map where the keys are atoms (typically representing backend names) and\nthe values are the configuration data from those backends.","ref":"Kungfuig.html#t:config/0"},{"type":"function","title":"Kungfuig.config/2","doc":"Retrieves the current config for the key(s) specified","ref":"Kungfuig.html#config/2"},{"type":"type","title":"Kungfuig.option/0","doc":"The options that can be passed to `start_link/1` function.\n\nOptions include:\n\n* `:name` - The name to register the Kungfuig instance under. Default is `Kungfuig`. \n  @since v0.4.0\n\n* `:callback` - One or more callbacks to notify when configuration changes. See `t:callback/0`.\n\n* `:interval` - How frequently to check for configuration updates, in milliseconds. \n  Default is 1000 (1 second).\n\n* `:imminent` - When `true`, performs configuration validation immediately during initialization,\n  rather than as a continuation after init returns. Default is `false`.\n\n* `:anonymous` - When `true`, the `GenServer` is not registered under a name.\n  Default is `false`.\n\n* `:start_options` - Options passed to `GenServer.start_link/3`.\n\n* `:validator` - A module implementing the `Kungfuig.Validator` behaviour to validate\n  the configuration. Default is `Kungfuig.Validators.Void` (no validation).\n  @since v0.3.0\n\n* `:workers` - A list of backend specifications, each either a module name or a \n  `{module, opts}` tuple. It might also be a function of arity","ref":"Kungfuig.html#t:option/0"},{"type":"function","title":"Kungfuig.start_link/0","doc":"The function to start the `Kungfuig` manager under a supervision tree with default options","ref":"Kungfuig.html#start_link/0"},{"type":"function","title":"Kungfuig.start_link/1","doc":"The function to start the `Kungfuig` manager under a supervision tree with custom options","ref":"Kungfuig.html#start_link/1"},{"type":"behaviour","title":"Kungfuig.Backend","doc":"The scaffold for the backend watching the external config source.","ref":"Kungfuig.Backend.html"},{"type":"behaviour","title":"Example - Kungfuig.Backend","doc":"```elixir\ndefmodule Kungfuig.Backends.EnvTransform do\n  @moduledoc false\n\n  use Kungfuig.Backend, report: :logger, imminent: true\n\n  @impl Kungfuig.Backend\n  def get(meta, key) do\n    {:ok,\n     meta\n     |> Keyword.get(:for, [key])\n     |> Enum.reduce(%{}, &Map.put(&2, &1, Application.get_all_env(&1)))}\n  end\n\n  @impl Kungfuig.Backend\n  def transform(%{kungfuig: env}) do\n    {:ok, for({k, v} <- env, {_, value} <- v, do: {k, value})}\n  end\nend\n```","ref":"Kungfuig.Backend.html#module-example"},{"type":"callback","title":"Kungfuig.Backend.get/1","doc":"The implementation of the call to remote that retrieves the data","ref":"Kungfuig.Backend.html#c:get/1"},{"type":"callback","title":"Kungfuig.Backend.get/2","doc":"The implementation of the call to remote that retrieves the data under a key specified","ref":"Kungfuig.Backend.html#c:get/2"},{"type":"callback","title":"Kungfuig.Backend.key/0","doc":"The key this particular config would be stored under, defaults to module name","ref":"Kungfuig.Backend.html#c:key/0"},{"type":"type","title":"Kungfuig.Backend.key/0","doc":"The type for the key withing `Kungfuig`, typically an atom.","ref":"Kungfuig.Backend.html#t:key/0"},{"type":"callback","title":"Kungfuig.Backend.report/1","doc":"The implementation of error reporting","ref":"Kungfuig.Backend.html#c:report/1"},{"type":"callback","title":"Kungfuig.Backend.transform/1","doc":"The transformer that converts the retrieved data to internal representation","ref":"Kungfuig.Backend.html#c:transform/1"},{"type":"behaviour","title":"Kungfuig.Callback","doc":"The behaviour defining the dynamic config provider’s callback.\n\n`Kungfuig ` accepts callbacks in several forms, see `t:Kungfuig.callback/0`.","ref":"Kungfuig.Callback.html"},{"type":"callback","title":"Kungfuig.Callback.handle_config_update/1","doc":"The callback to be used for subscribing to config updates.","ref":"Kungfuig.Callback.html#c:handle_config_update/1"},{"type":"module","title":"Kungfuig.Parser","doc":"The parser for the value read from a config.\n\nIt is designed to simplify reading and instantiating values read from json\n  to elixir entities.\n\nSupported types:\n\n- `{\"type\": \"atom\", \"value\": \"foo\"}` — the atom `:foo`\n- `{\"type\": \"existing_atom\", \"value\": \"foo\"}` — the existing atom `:foo`\n- `{\"type\": \"function\", \"module\": module, \"function\": function, \"arity\": arity}` — the function to instantiate\n- `{\"type\": \"existing_function\", \"module\": module, \"function\": function, \"arity\": arity}` — the existing function to instantiate\n- `{\"type\": \"tuple\", \"value\": [\"foo\", 42]}` — the tuple `{\"foo\", 42}`","ref":"Kungfuig.Parser.html"},{"type":"function","title":"Kungfuig.Parser.value/1","doc":"Retrieves and instantiates the actial value out of json object","ref":"Kungfuig.Parser.html#value/1"},{"type":"behaviour","title":"Kungfuig.Validator","doc":"Generic validator encapsulation.\n\nDefault implementation is\n  [`NimbleOptions`](https://github.com/dashbitco/nimble_options).","ref":"Kungfuig.Validator.html"},{"type":"behaviour","title":"Example - Kungfuig.Validator","doc":"```elixir\ndefmodule TemperatureValidator do\n  @moduledoc false\n\n  use Kungfuig.Validator, schema: [\n    city: [type: :string, required: true],\n    temperature: [type: :integer, required: false]\n  ]\nend\n```","ref":"Kungfuig.Validator.html#module-example"},{"type":"callback","title":"Kungfuig.Validator.doc/1","doc":"Generates a documentation for expected options","ref":"Kungfuig.Validator.html#c:doc/1"},{"type":"callback","title":"Kungfuig.Validator.validate/1","doc":"Validates given options with the schema, provided by the implementation of this behaviour","ref":"Kungfuig.Validator.html#c:validate/1"},{"type":"extras","title":"Kungfuig","doc":"# ![Kungfuig](https://raw.githubusercontent.com/kantox/kungfuig/master/stuff/kungfuig-48x48.png) Kungfuig    [![Kantox ❤ OSS](https://img.shields.io/badge/❤-kantox_oss-informational.svg)](https://kantox.com/)  ![Test](https://github.com/kantox/kungfuig/workflows/Test/badge.svg)  ![Dialyzer](https://github.com/kantox/kungfuig/workflows/Dialyzer/badge.svg)","ref":"readme.html"},{"type":"extras","title":"Intro - Kungfuig","doc":"Live config supporting many different backends.\n\n**Kungfuig** (_pronounced:_ [ˌkʌŋˈfig]) provides an easy way to plug\nlive configuration into everything.\n\nIt provides backends for `env` and `system` and supports custom backends. Configurations are periodically updated, and changes can trigger callback notifications to relevant application components.","ref":"readme.html#intro"},{"type":"extras","title":"Installation - Kungfuig","doc":"```elixir\ndef deps do\n  [\n    {:kungfuig, \"~> 1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Architecture - Kungfuig","doc":"Kungfuig is designed with a robust architecture that provides flexibility and reliability:\n\n![Kungfuig Architecture](https://raw.githubusercontent.com/kantox/kungfuig/master/stuff/kungfuig.drawio.png)\n\nThe architecture consists of several key components:\n\n1. **Supervisor** (`Kungfuig.Supervisor`): Manages the configuration process tree\n2. **Manager** (`Kungfuig.Manager`): Supervises the backend workers\n3. **Backend Workers** (`Kungfuig.Backend` implementations): Poll configuration sources\n4. **Blender** (`Kungfuig.Blender`): Combines configurations from all backends\n5. **Callbacks**: Notify application components when configurations change\n6. **Validators**: Ensure configuration values match expected schemas\n\nThis architecture provides several benefits:\n- Fault tolerance through supervisor patterns\n- Separation of concerns between configuration sources\n- Unified configuration access for your application\n- Pluggable backends for custom configuration sources","ref":"readme.html#architecture"},{"type":"extras","title":"Using - Kungfuig","doc":"","ref":"readme.html#using"},{"type":"extras","title":"Basic Usage - Kungfuig","doc":"**Kungfuig** is the easy way to read the external configuration from sources that are not controlled by the application using it, such as _Redis_, or _Database_.\n\n```elixir\n# Start Kungfuig with default options\nKungfuig.start_link()\n\n# Get all configuration\nKungfuig.config()\n#⇒ %{env: %{kungfuig: []}, system: %{}}\n\n# Get configuration from a specific backend\nKungfuig.config(:env)\n#⇒ %{kungfuig: []}\n\n# Update application environment and see the change\nApplication.put_env(:kungfuig, :foo, 42)\nKungfuig.config(:env)\n#⇒ %{kungfuig: [foo: 42]}\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"Custom Backends - Kungfuig","doc":"Here is an example of a backend implementation for reading configuration from an external MySQL database:\n\n```elixir\ndefmodule MyApp.Kungfuig.MySQL do\n  @moduledoc false\n\n  use Kungfuig.Backend, interval: 300_000 # 5 minutes\n\n  @impl Kungfuig.Backend\n  def get(_meta) do\n    with {:ok, host} <- System.fetch_env(\"MYSQL_HOST\"),\n         {:ok, db} <- System.fetch_env(\"MYSQL_DB\"),\n         {:ok, user} <- System.fetch_env(\"MYSQL_USER\"),\n         {:ok, pass} <- System.fetch_env(\"MYSQL_PASS\"),\n         {:ok, pid} when is_pid(pid) <-\n           MyXQL.start_link(hostname: host, database: db, username: user, password: pass),\n         result <- MyXQL.query!(pid, \"SELECT * FROM some_table\") do\n      GenServer.stop(pid)\n\n      result =\n        result.rows\n        |> Flow.from_enumerable()\n        |> Flow.map(fn [_, field1, field2, _, _] -> {field1, field2} end)\n        |> Flow.partition(key: &elem(&1, 0))\n        |> Flow.reduce(fn -> %{} end, fn {field1, field2}, acc ->\n          Map.update(\n            acc,\n            String.to_existing_atom(field1),\n            [field2],\n            &[field2 | &1]\n          )\n        end)\n\n      Logger.info(\"Loaded #{Enum.count(result)} values from \" <> host)\n\n      {:ok, result}\n    else\n      :error ->\n        Logger.warn(\"Skipped reconfig, one of MYSQL_{HOST,DB,USER,PASS} is missing\")\n        :ok\n\n      error ->\n        Logger.error(\"Reconfiguring failed. Error: \" <> inspect(error))\n        {:error, error}\n    end\n  end\nend\n```","ref":"readme.html#custom-backends"},{"type":"extras","title":"Callback Patterns - Kungfuig","doc":"Kungfuig supports multiple callback mechanisms to notify your application when configuration changes:\n\n#### Process Message Callback\n\n```elixir\n# Using a PID - the process will receive a {:kungfuig_update, config} message\ndefmodule MyApp.ConfigConsumer do\n  use GenServer\n  \n  def start_link(_), do: GenServer.start_link(__MODULE__, nil)\n  \n  def init(_), do: {:ok, %{}}\n\n  # Handle configuration updates\n  def handle_info({:kungfuig_update, config}, state) do\n    IO.puts(\"Configuration updated: #{inspect(config)}\")\n    {:noreply, state}\n  end\nend\n\n# In your application startup\n{:ok, pid} = MyApp.ConfigConsumer.start_link([])\nKungfuig.start_link(workers: [{Kungfuig.Backends.Env, callback: pid}])\n```\n\n#### Callback Behaviour Implementation\n\n```elixir\ndefmodule MyApp.ConfigHandler do\n  @behaviour Kungfuig.Callback\n  \n  @impl true\n  def handle_config_update(config) do\n    IO.puts(\"Configuration updated: #{inspect(config)}\")\n    :ok\n  end\nend\n\n# In your application startup\nKungfuig.start_link(workers: [{Kungfuig.Backends.Env, callback: MyApp.ConfigHandler}])\n```\n\n#### GenServer Interaction Callbacks\n\n```elixir\n# Using a GenServer cast\ndefmodule MyApp.ConfigManager do\n  use GenServer\n\n  def start_link(_), do: GenServer.start_link(__MODULE__, nil, name: __MODULE__)\n  \n  def init(_), do: {:ok, %{}}\n\n  def handle_cast({:config_updated, config}, state) do\n    # Process configuration update\n    {:noreply, Map.put(state, :config, config)}\n  end\nend\n\n# In your application startup\nMyApp.ConfigManager.start_link([])\nKungfuig.start_link(\n  workers: [\n    {Kungfuig.Backends.Env, callback: {MyApp.ConfigManager, {:cast, :config_updated}}}\n  ]\n)\n```\n\n#### Function Callback\n\n```elixir\n# Using an anonymous function\ncallback_fn = fn config ->\n  IO.puts(\"Config updated: #{inspect(config)}\")\n  :ok\nend\n\nKungfuig.start_link(workers: [{Kungfuig.Backends.Env, callback: callback_fn}])\n```","ref":"readme.html#callback-patterns"},{"type":"extras","title":"Validation System - Kungfuig","doc":"Since v0.3.0, Kungfuig supports configuration validation through the `Kungfuig.Validator` behavior, which by default uses `NimbleOptions` for schema validation.\n\n#### Creating a Custom Validator\n\n```elixir\ndefmodule MyApp.ConfigValidator do\n  use Kungfuig.Validator, schema: [\n    database: [\n      type: :string,\n      required: true,\n      doc: \"The database name\"\n    ],\n    hostname: [\n      type: :string,\n      required: true,\n      doc: \"The database hostname\"\n    ],\n    port: [\n      type: :integer,\n      default: 5432,\n      doc: \"The database port\"\n    ],\n    ssl: [\n      type: :boolean,\n      default: false,\n      doc: \"Whether to use SSL\"\n    ]\n  ]\nend\n```\n\n#### Using the Validator\n\nYou can apply validators at two levels:\n\n1. **Per backend** - to validate only this backend's configuration:\n\n```elixir\nKungfuig.start_link(\n  workers: [\n    {MyApp.Kungfuig.DatabaseBackend, validator: MyApp.ConfigValidator}\n  ]\n)\n```\n\n2. **Global validator** - to validate the entire configuration:\n\n```elixir\nKungfuig.start_link(\n  validator: MyApp.GlobalConfigValidator,\n  workers: [\n    MyApp.Kungfuig.DatabaseBackend,\n    MyApp.Kungfuig.RedisBackend\n  ]\n)\n```","ref":"readme.html#validation-system"},{"type":"extras","title":"Named Instances - Kungfuig","doc":"Since v0.4.0, Kungfuig supports multiple named instances, allowing different parts of your application to use separate configuration managers.\n\n```elixir\n# Start a Kungfuig instance for database configuration\nKungfuig.start_link(\n  name: MyApp.DBConfig,\n  workers: [\n    {MyApp.Kungfuig.DatabaseBackend, interval: 60_000}\n  ]\n)\n\n# Start another Kungfuig instance for API configuration\nKungfuig.start_link(\n  name: MyApp.APIConfig,\n  workers: [\n    {MyApp.Kungfuig.APIBackend, interval: 30_000}\n  ]\n)\n\n# Query the configurations separately\ndb_config = Kungfuig.config(:database, MyApp.DBConfig)\napi_config = Kungfuig.config(:api, MyApp.APIConfig)\n```","ref":"readme.html#named-instances"},{"type":"extras","title":"Immediate Configuration Processing - Kungfuig","doc":"Since v0.4.2, Kungfuig allows immediate configuration validation using the `imminent: true` option:\n\n```elixir\nKungfuig.start_link(\n  workers: [\n    {Kungfuig.Backends.Env, imminent: true}\n  ]\n)\n```\n\nThis option ensures that the configuration is validated and processed during the init phase, rather than as a continuation after init returns. This is useful when your application requires the configuration to be available immediately upon startup.","ref":"readme.html#immediate-configuration-processing"},{"type":"extras","title":"Testing - Kungfuig","doc":"Simply implement a stub returning an expected config and you are all set.\n\n```elixir\ndefmodule MyApp.Kungfuig.Stub do\n  @moduledoc false\n\n  use Kungfuig.Backend\n\n  @impl Kungfuig.Backend\n  def get(_meta), do: %{foo: :bar, baz: [42]}\nend\n```","ref":"readme.html#testing"},{"type":"extras","title":"Comparison with Alternatives - Kungfuig","doc":"Kungfuig offers several advantages compared to other configuration management solutions:","ref":"readme.html#comparison-with-alternatives"},{"type":"extras","title":"vs. Application Environment - Kungfuig","doc":"The standard Elixir Application environment is static after startup unless manually changed:\n\n- **Kungfuig**: Automatically polls for changes at configurable intervals\n- **App env**: Requires manual calls to `Application.put_env/3` to update","ref":"readme.html#vs-application-environment"},{"type":"extras","title":"vs. Config Providers - Kungfuig","doc":"Elixir's built-in config providers run only once at application start:\n\n- **Kungfuig**: Continually monitors for changes while the application is running\n- **Config providers**: Only run during application startup","ref":"readme.html#vs-config-providers"},{"type":"extras","title":"vs. Distillery Config Providers - Kungfuig","doc":"While Distillery's Config Providers can load configuration on application start:\n\n- **Kungfuig**: Lets you define custom backends for any data source\n- **Distillery**: Focused on files and environment variables\n- **Kungfuig**: Supports multiple callback types for change notifications\n- **Distillery**: Doesn't have built-in change notification","ref":"readme.html#vs-distillery-config-providers"},{"type":"extras","title":"vs. etcd/Consul/ZooKeeper clients - Kungfuig","doc":"While these external services provide great configuration management:\n\n- **Kungfuig**: Works with many different backends, not tied to a specific service\n- **External services**: Require running and maintaining additional infrastructure\n- **Kungfuig**: Simple supervision tree model familiar to Elixir developers\n- **External services**: More complex client libraries and connection management","ref":"readme.html#vs-etcd-consul-zookeeper-clients"},{"type":"extras","title":"Changelog - Kungfuig","doc":"- **`1.0.0`** — modern Elixir v1.16\n- **`0.4.4`** — fix a bug with hardcoded names (`Supervisor` and `Blender`)\n- **`0.4.2`** — allow `imminent: true` option to `Kungfuig.Backend`\n- **`0.4.0`** — allow named `Kungfuig` instances (thanks @vbroskas)\n- **`0.3.0`** — allow validation through `NimbleOptions` (per backend and global)\n- **`0.2.0`** — scaffold for backends + several callbacks (and the automatic one for `Blender`)\n\n## [Documentation](https://hexdocs.pm/kungfuig)","ref":"readme.html#changelog"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}